package com.ruoyi.screen.config;

import com.rabbitmq.client.Channel;
import com.ruoyi.common.redis.service.RedisService;
import com.ruoyi.screen.contones.RedisConstant;
import com.ruoyi.screen.contones.VehicleConstant;
import com.ruoyi.screen.domain.FaultsLog;
import com.ruoyi.screen.domain.VehicleFaults;
import com.ruoyi.screen.mapper.CarMapper;
import com.ruoyi.screen.service.CarService;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.BoundSetOperations;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;

/**
 * Rabbit消费者
 */
@Component
@Log4j2
public class RabbitConfig {

    @Autowired
    private RabbitTemplate rabbitTemplate;
    @Autowired
    private RedisService redisService;
    @Autowired
    private CarMapper carMapper;
    @Autowired
    private CarService carService;

    @RabbitListener(queuesToDeclare = {@Queue("fault_handler_queue")})
    public void RabbitAccept(String exeMessage, Channel channel, Message message) {
        try {

            String messageId = message.getMessageProperties().getMessageId();

            HashSet<String> strings = new HashSet<>();
            strings.add(messageId);

            BoundSetOperations<String, String> faultHandlerQueue = redisService.setCacheSet("fault_handler_queue", strings);
            Long size = faultHandlerQueue.size();
            if(size>0){
                redisService.deleteObject("fault_handler_queue");
                log.info("接受到的报文消息消息:"+exeMessage);
                //取出故障码
                ArrayList<VehicleFaults> arrayList = (ArrayList<VehicleFaults>) redisService.getCacheObject(RedisConstant.FAULTS_MESSAGE);
                //日志记录集合
                ArrayList<FaultsLog> faultsLogs = new ArrayList<>();
                //循环所有故障码比对
                for (VehicleFaults vehicleFaults : arrayList) {
                    //取出故障位
                    Integer faultBit = vehicleFaults.getFaultBit();
                    //截取vin
                    String getvin = getvin(exeMessage);
                    //截取时间戳

                    Long time = getTime(exeMessage);
                    //判断故障位是否有故障

                    if(Integer.parseInt(exeMessage.substring(faultBit,faultBit+1))==0){
                        //取redis日志数据
                        FaultsLog faultsLog = (FaultsLog) redisService.getCacheObject(RedisConstant.FAULTS_LOG + getvin + vehicleFaults.getFaultCode());
                        if(faultsLog==null){
                            FaultsLog faultsLog1 = new FaultsLog();
                            faultsLog1.setFaultId(vehicleFaults.getFaultId());
                            faultsLog1.setFaultCode(vehicleFaults.getFaultCode());
                            faultsLog1.setFaultVin(getvin);
                            faultsLog1.setLogStart(new Date(time));
                            //数据存入redis
                            redisService.setCacheObject(RedisConstant.FAULTS_LOG+getvin+vehicleFaults.getFaultCode(),faultsLog1);
                            //记录在故障记录表
                            faultsLogs.add(faultsLog1);
                        }

                        carService.sendEmail(faultsLog);

                    } else {
                        //取出故障记录
                        FaultsLog faultsLog = (FaultsLog) redisService.getCacheObject(RedisConstant.FAULTS_LOG + getvin + vehicleFaults.getFaultCode());
                        if(faultsLog!=null){
                            faultsLog.setLogEnd(new Date(time));
                            //根据vin以及故障码查找出最大的id
                            faultsLog.setLogId(carMapper.findFaultId(faultsLog));
                            //对这条数据进行修改报警结束的时间
                            carMapper.updateFaults(faultsLog);
                            //删除redis中存储的记录
                           redisService.deleteObject(RedisConstant.FAULTS_LOG + getvin + vehicleFaults.getFaultCode());
                        }
                    }
                }

                if(!faultsLogs.isEmpty()){
                    //先取redis看有没有值
                    Object day = redisService.getCacheObject(RedisConstant.VEHICLE_INFO_TOKEN + RedisConstant.CURRENT_DAY_FAULT);
                    Object month = redisService.getCacheObject(RedisConstant.VEHICLE_INFO_TOKEN + RedisConstant.CURRENT_MONTH_FAULT);
                    if(day!=null){
                        redisService.setCacheObject(RedisConstant.VEHICLE_INFO_TOKEN+RedisConstant.CURRENT_DAY_FAULT,Integer.valueOf(day.toString())+faultsLogs.size());
                    }else {
                        redisService.setCacheObject(RedisConstant.VEHICLE_INFO_TOKEN+RedisConstant.CURRENT_DAY_FAULT,faultsLogs.size());
                    }

                    if(month!=null){
                        redisService.setCacheObject(RedisConstant.VEHICLE_INFO_TOKEN+RedisConstant.CURRENT_MONTH_FAULT,Integer.valueOf(month.toString())+faultsLogs.size());
                    }else {
                        redisService.setCacheObject(RedisConstant.VEHICLE_INFO_TOKEN+RedisConstant.CURRENT_MONTH_FAULT,faultsLogs.size());
                    }
                    for (FaultsLog faultsLog : faultsLogs) {
                        Date logStart = faultsLog.getLogStart();
                        faultsLog.setLogStart(logStart);
                        //将数据存入数据库
                        carMapper.addFaults(faultsLog);
                    }

                }

                //确认接受MQ消息
                channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
                log.info("队列名称:{},队列参数:{},消费成功......","fault_handler_queue",exeMessage);
            }else {
                log.info("队列名称:{},队列参数:{},消费重复......","fault_handler_queue",exeMessage);
            }
        } catch (Exception e) {
            e.printStackTrace();
            try {
                log.info("队列名称:{},队列参数:{},消费异常......","fault_handler_queue",exeMessage);
                try {
                    channel.basicNack(message.getMessageProperties().getDeliveryTag(),true,false);
                } catch (IOException ioException) {
                    ioException.printStackTrace();
                }
            } catch (Exception exception) {
                exception.printStackTrace();
            }
        }

    }



    /**
     * 截取vin
     * @param message
     * @return
     */

    public static  String getvin(String message){
        //关闭连接信息
        if(message.contains(VehicleConstant.NETTY_WILL_CLOSE)){
            return message.substring(VehicleConstant.NETTY_WILL_CLOSE.length(),VehicleConstant.NETTY_WILL_CLOSE.length()+17);
        }
        //连接成功信息
        if(message.contains(VehicleConstant.NETTY_CONNECT)){
            return message.substring(VehicleConstant.NETTY_CONNECT.length(),VehicleConstant.NETTY_CONNECT.length()+17);
        }
        //车辆启动
        if(message.contains(VehicleConstant.START_VIN_SUCCESS_SUF)){
            return message.substring(VehicleConstant.START_VIN_SUCCESS_SUF.length(),VehicleConstant.START_VIN_SUCCESS_SUF.length()+17);
        }

        //车辆停止
        if(message.contains(VehicleConstant.VEHICLE_STOP_SUF)){
            return message.substring(VehicleConstant.VEHICLE_STOP_SUF.length(),VehicleConstant.VEHICLE_STOP_SUF.length()+17);
        }
        //车辆消息
        if(message.contains(VehicleConstant.VEHICLE_MSG_SUF)){
            return message.substring(VehicleConstant.VEHICLE_MSG_SUF.length()+1,VehicleConstant.VEHICLE_MSG_SUF.length()+18);
        }

        return "未找到对应的信息 参数为:"+message;
    }

    /**
     * 截取时间戳
     * @param message
     * @return
     */
    public static  Long getTime(String message){
        Long substring = Long.parseLong(message.substring(206,219));
        return substring;
    }
}
