package com.ruoyi.screen.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.ruoyi.common.redis.service.RedisService;
import com.ruoyi.screen.contones.RFaults;
import com.ruoyi.screen.domain.FaultsLog;
import com.ruoyi.screen.domain.Type;
import com.ruoyi.screen.domain.VehicleFaults;
import com.ruoyi.screen.mapper.CarMapper;
import com.ruoyi.screen.result.PageResult;
import com.ruoyi.screen.result.Result;
import com.ruoyi.screen.service.CarService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Service;

import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;



@Service
@Log4j2
public class CarServiceimpl implements CarService {

    @Autowired
    private CarMapper carMapper;
    @Autowired
    private RedisService redisService;
    @Autowired
    private JavaMailSender javaMailSender;

    @Override
    public ArrayList<VehicleFaults> FaultsMessage() {
        return carMapper.FaultsMessage();
    }

    @Override
    public Result AddFaultsCode(VehicleFaults vehicleFaults) {
        carMapper.AddFaultsCode(vehicleFaults);
        return Result.success("添加成功");
    }

    @Override
    public List<Type> FindType() {
        return carMapper.FindType();
    }

    @Override
    public Result deleteFaultsCode(Integer faultId) {

        int i=  carMapper.deleteFaultsCode(faultId);

        return Result.success("删除成功");
    }

    @Override
    public VehicleFaults findFaultsId(Integer faultId) {
        return carMapper.findFaultsId(faultId);
    }

    @Override
    public Result<PageResult<VehicleFaults>> findFault(RFaults rFaults) {
        PageHelper.startPage(rFaults.getPageNum(),rFaults.getPageSize());

        List<VehicleFaults> vehicleFaults=carMapper.findFault(rFaults);

        PageInfo<VehicleFaults> vehicleFaultsPageInfo = new PageInfo<>(vehicleFaults);

        return PageResult.toResult(vehicleFaultsPageInfo.getTotal(),vehicleFaults);
    }

    @Override
    public Result faultUpdate(VehicleFaults vehicleFaults) {
        carMapper.faultUpdate(vehicleFaults);

        return Result.success("修改成功");
    }

    @Override
    public void sendEmail(FaultsLog faultsLog) {
        if(faultsLog!=null){
            FaultsLog log1 = carMapper.findLog(faultsLog.getFaultVin(), faultsLog.getLogStart());
            if(redisService.hasKey("errorCar_"+log1.getLogId())){
                log.info("邮件已经发送，请等到该车主回执："+log1.getFaultVin());
            }else{
                try {
                    MimeMessage mimeMessage = javaMailSender.createMimeMessage();
                    MimeMessageHelper helper = new MimeMessageHelper(mimeMessage, true);
                    helper.setSubject("通知");
                    helper.setTo("3079188394@qq.com");
                    helper.setText("您的车辆故障，故障码是："+faultsLog.getFaultCode()+"，故障车辆是："+faultsLog.getFaultVin());
                    helper.setFrom("3079188394@qq.com");
                    javaMailSender.send(mimeMessage);
                    redisService.setCacheObject("errorCar_"+log1.getLogId(),"", 5L,TimeUnit.MINUTES);
                } catch (MessagingException e) {
                    throw new RuntimeException(e);
                }
            }
        }

    }

    @Override
    public Result<List<Map<Object,Object>>> groupCount() {
        List<Map<Object, Object>> maps = carMapper.groupCount();
        return Result.success(maps);
    }

    @Override
    public Result<List<Map<Object, Object>>> toDay() {
        List<Map<Object, Object>> day = carMapper.toDay();
        return Result.success(day);
    }

    @Override
    public Result<List<Map<Object, Object>>> yestDay() {
        List<Map<Object, Object>> maps = carMapper.yestDay();
        return Result.success(maps);
    }
}
